@@ -668,16 +668,25 @@ WindowText				= 26,*/
 		
 			case Win32.Api.WindowsMessages.WM_KEYDOWN:
 			case Win32.Api.WindowsMessages.WM_SYSKEYDOWN:
-				DrawingWindow(hwnd).KeyDown( wParam, lParam );
+				if (!DrawingWindow(hwnd).KeyDown( wParam, lParam ))
+				{
+					retval = Win32.Api.DefWindowProcA(hwnd, msg, wParam, lParam);
+				}
 				break;
 			case Win32.Api.WindowsMessages.WM_CHAR:
-				DrawingWindow(hwnd).Char( wParam, lParam );
+			case Win32.Api.WindowsMessages.WM_SYSCHAR:
+				if (!DrawingWindow(hwnd).Char( wParam, lParam ))
+				{
+					retval = Win32.Api.DefWindowProcA(hwnd, msg, wParam, lParam);
+				}
 				break;
 			case Win32.Api.WindowsMessages.WM_KEYUP:
 			case Win32.Api.WindowsMessages.WM_SYSKEYUP:
-				DrawingWindow(hwnd).KeyUp( wParam, lParam );
+				if (!DrawingWindow(hwnd).KeyUp( wParam, lParam ))
+				{
+					retval = Win32.Api.DefWindowProcA(hwnd, msg, wParam, lParam);
+				}
 				break;
-
 			case Win32.Api.WindowsMessages.WM_SETTINGCHANGE:
 				DrawingWindow(hwnd).SettingsChange( wParam );
 				break;
@@ -751,6 +760,24 @@ WindowText				= 26,*/
 		return null;
 	}
 
+	public IToolkitWindowBuffer CreateWindowBuffer(IToolkitWindow window)
+	{
+		return new DrawingWindowBuffer(window);
+	}
+
+	internal static IntPtr RectanglesToRegion(Rectangle[] rects)
+	{
+		IntPtr region = Win32.Api.CreateRectRgn(0,0,0,0);
+		IntPtr region1 = Win32.Api.CreateRectRgn(0,0,0,0);
+		for (int i = 0; i < rects.Length; i++)
+		{
+			Rectangle rect = rects[i];
+			Win32.Api.SetRectRgn(region1, rect.Left, rect.Top, rect.Right, rect.Bottom);
+			Win32.Api.CombineRgn(region, region, region1, Win32.Api.RegionCombineMode.RGN_OR);
+		}
+		Win32.Api.DeleteObject(region1);
+		return region;
+	}
 
 }; // class DrawingToolkit
 
