@@ -174,7 +174,14 @@ internal sealed class DrawingMdiClient : MdiClientWidget, IToolkitMdiClient
 	// Set the background of the window to a solid color.
 	void IToolkitWindow.SetBackground(System.Drawing.Color color)
 			{
-				Background = DrawingToolkit.DrawingToXColor(color);
+				if(color.A < 128)
+				{
+					BackgroundPixmap = null;
+				}
+				else
+				{
+					Background = DrawingToolkit.DrawingToXColor(color);
+				}
 			}
 
 	// Move this window to above one of its siblings.
