@@ -37,7 +37,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	// Internal state.
 	private IToolkitGraphics graphics;
 	private Region clip;
-	private Matrix transform;
+	internal Matrix transform;
 	private float pageScale;
 	private GraphicsUnit pageUnit;
 	internal GraphicsContainer stackTop;
@@ -67,7 +67,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	// Graphics that has an origin of baseWindow.Location and is always clipped
 	// to baseWindow
 
-	public Graphics(IToolkitGraphics graphics, Rectangle baseWindow)
+	internal Graphics(IToolkitGraphics graphics, Rectangle baseWindow)
 		: this(graphics)
 			{
 				this.baseWindow = baseWindow;
@@ -77,7 +77,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 			}
 
 	// Create a Graphics that is internally offset to baseWindow
-	public Graphics(Graphics graphics, Rectangle baseWindow)
+	internal Graphics(Graphics graphics, Rectangle baseWindow)
 			{
 				// Use the same toolkit
 				this.graphics = graphics.graphics;
@@ -121,6 +121,11 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 						graphics.Dispose();
 						graphics = null;
 					}
+					if (clip != null)
+					{
+						clip.Dispose();
+						clip = null;
+					}
 				}
 			}
 
@@ -524,9 +529,9 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	// Clear the entire drawing surface.
 	public void Clear(Color color)
 			{
-				lock(this)
+				using (Brush brush = new SolidBrush(color))
 				{
-					ToolkitGraphics.Clear(color);
+					FillRectangle(brush, ClipBounds);
 				}
 			}
 
@@ -785,33 +790,92 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				}
 			}
 
-	// Draw an icon.
-	[TODO]
-	public void DrawIcon(Icon icon, Rectangle targetRect)
+	// Draw an unstretched complete icon via the toolkit.
+	private void ToolkitDrawIcon(Icon icon, int x, int y)
 			{
-				// TODO
-				DrawIconUnstretched(icon, targetRect);
-			}
-	[TODO]
-	public void DrawIconUnstretched(Icon icon, Rectangle targetRect)
-			{
-				// TODO
-				if(icon == null)
-				{
-					throw new ArgumentNullException("icon");
-				}
 				lock(this)
 				{
 					IToolkitImage toolkitImage = icon.GetToolkitImage(this);
 					if(toolkitImage != null)
 					{
-						int x = targetRect.X;
-						int y = targetRect.Y;
 						ConvertPoint(ref x, ref y, pageUnit);
-						x += baseWindow.X;
-						y += baseWindow.Y;
 						ToolkitGraphics.DrawImage
-							(toolkitImage, x, y);
+							(toolkitImage, x + baseWindow.X,
+							 y + baseWindow.Y);
+					}
+				}
+			}
+
+	// Draw an icon.
+	public void DrawIcon(Icon icon, Rectangle targetRect)
+			{
+				if(icon == null)
+				{
+					throw new ArgumentNullException("icon");
+				}
+				if(targetRect.Width == icon.Width &&
+				   targetRect.Height == icon.Height)
+				{
+					// This is the easy case.
+					ToolkitDrawIcon(icon, targetRect.X, targetRect.Y);
+				}
+				else
+				{
+					// Stretch and draw the image.
+					Point[] src = ConvertRectangle3
+						(0, 0, icon.Width, icon.Height, GraphicsUnit.Pixel);
+					Point[] dest = ConvertRectangle3
+						(targetRect.X, targetRect.Y,
+						 targetRect.Width, targetRect.Height, pageUnit);
+					BaseOffsetPoints(dest);
+					lock(this)
+					{
+						IToolkitImage toolkitImage = icon.GetToolkitImage(this);
+						if(toolkitImage != null)
+						{
+							ToolkitGraphics.DrawImage
+								(toolkitImage, src, dest);
+						}
+					}
+				}
+			}
+	public void DrawIconUnstretched(Icon icon, Rectangle targetRect)
+			{
+				if(icon == null)
+				{
+					throw new ArgumentNullException("icon");
+				}
+				int width = icon.Width;
+				int height = icon.Height;
+				if(width <= targetRect.Width && height <= targetRect.Height)
+				{
+					// No truncation is necessary, so use the easy case.
+					ToolkitDrawIcon(icon, targetRect.X, targetRect.Y);
+				}
+				else
+				{
+					// Only draw as much as will fit in the target rectangle.
+					if(width > targetRect.Width)
+					{
+						width = targetRect.Width;
+					}
+					if(height > targetRect.Height)
+					{
+						height = targetRect.Height;
+					}
+					Point[] src = ConvertRectangle3
+						(0, 0, width, height, GraphicsUnit.Pixel);
+					Point[] dest = ConvertRectangle3
+						(targetRect.X, targetRect.Y, width, height, pageUnit);
+					BaseOffsetPoints(dest);
+					lock(this)
+					{
+						IToolkitImage toolkitImage = icon.GetToolkitImage(this);
+						if(toolkitImage != null)
+						{
+							ToolkitGraphics.DrawImage
+								(toolkitImage, src, dest);
+						}
 					}
 				}
 			}
@@ -821,8 +885,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				{
 					throw new ArgumentNullException("icon");
 				}
-				DrawIconUnstretched
-					(icon, new Rectangle(x, y, icon.Width, icon.Height));
+				ToolkitDrawIcon(icon, x, y);
 			}
 
 	// Used to call the toolkit DrawImage methods
@@ -831,10 +894,6 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				if (image.toolkitImage == null)
 					image.toolkitImage = ToolkitGraphics.Toolkit.CreateImage(image.dgImage, 0);
 				BaseOffsetPoints(dest);
-				// width and height must be 1 bigger.
-				dest[1].X += 1;
-				dest[2].Y += 1;
-				
 				ToolkitGraphics.DrawImage(image.toolkitImage,src, dest);
 			}
 
@@ -946,30 +1005,11 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				dest[2].Y = image.Height;
 				ToolkitDrawImage(image, src, dest);
 			}
-	[TODO]
-	public void DrawImage(Image image, Point[] destPoints,
-						  Rectangle srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-	[TODO]
-	public void DrawImage(Image image, PointF[] destPoints,
-						  RectangleF srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-	[TODO]
 	public void DrawImage(Image image, float x, float y,
 						  RectangleF srcRect, GraphicsUnit srcUnit)
 			{
 				Point[] src = ConvertRectangle3(srcRect.X, srcRect.Y, srcRect.Width, srcRect.Height, srcUnit);
-				Point[] dest = ConvertRectangle3(x, y, 0, 0, pageUnit);
-				dest[1].X = image.Width;
-				dest[2].Y = image.Height;
+				Point[] dest = ConvertRectangle3(x, y, srcRect.Width, srcRect.Height, pageUnit);
 				ToolkitDrawImage(image, src, dest);
 			}
 	public void DrawImage(Image image, float x, float y,
@@ -979,43 +1019,6 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				Point[] dest = ConvertRectangle3(x, y, width, height, pageUnit);
 				ToolkitDrawImage(image, src, dest);
 			}
-	[TODO]
-	public void DrawImage(Image image, Point[] destPoints,
-						  Rectangle srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr,
-						  DrawImageAbort callback)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-	[TODO]
-	public void DrawImage(Image image, PointF[] destPoints,
-						  RectangleF srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr,
-						  DrawImageAbort callback)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-	[TODO]
-	public void DrawImage(Image image, Point[] destPoints,
-						  Rectangle srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr,
-						  DrawImageAbort callback, int callbackdata)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-	[TODO]
-	public void DrawImage(Image image, PointF[] destPoints,
-						  RectangleF srcRect, GraphicsUnit srcUnit,
-						  ImageAttributes imageAttr,
-						  DrawImageAbort callback, int callbackdata)
-			{
-				// TODO
-				DrawImage(image, destPoints, srcRect, srcUnit);
-			}
-
 	public void DrawImage(Image image, Rectangle destRect,
 						  int srcX, int srcY, int srcWidth, int srcHeight,
 						  GraphicsUnit srcUnit)
@@ -1034,60 +1037,97 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				Point[] dest = ConvertRectangle3(destRect.X, destRect.Y, destRect.Width, destRect.Height, pageUnit);
 				ToolkitDrawImage(image, src, dest);
 			}
-	[TODO]
+
+	// We don't use image attributes or image abort procedures in this
+	// implementation, so stub out the remaining DrawImage methods.
+	public void DrawImage(Image image, Point[] destPoints,
+						  Rectangle srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
+	public void DrawImage(Image image, PointF[] destPoints,
+						  RectangleF srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
+	public void DrawImage(Image image, Point[] destPoints,
+						  Rectangle srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr,
+						  DrawImageAbort callback)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
+	public void DrawImage(Image image, PointF[] destPoints,
+						  RectangleF srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr,
+						  DrawImageAbort callback)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
+	public void DrawImage(Image image, Point[] destPoints,
+						  Rectangle srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr,
+						  DrawImageAbort callback, int callbackdata)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
+	public void DrawImage(Image image, PointF[] destPoints,
+						  RectangleF srcRect, GraphicsUnit srcUnit,
+						  ImageAttributes imageAttr,
+						  DrawImageAbort callback, int callbackdata)
+			{
+				DrawImage(image, destPoints, srcRect, srcUnit);
+			}
 	public void DrawImage(Image image, Rectangle destRect,
 						  int srcX, int srcY, int srcWidth, int srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
-	[TODO]
 	public void DrawImage(Image image, Rectangle destRect,
 						  float srcX, float srcY,
 						  float srcWidth, float srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
-	[TODO]
 	public void DrawImage(Image image, Rectangle destRect,
 						  int srcX, int srcY, int srcWidth, int srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr,
 						  DrawImageAbort callback)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
-	[TODO]
 	public void DrawImage(Image image, Rectangle destRect,
 						  float srcX, float srcY,
 						  float srcWidth, float srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr,
 						  DrawImageAbort callback)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
-	[TODO]
 	public void DrawImage(Image image, Rectangle destRect,
 						  int srcX, int srcY, int srcWidth, int srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr,
 						  DrawImageAbort callback, IntPtr callbackData)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
-	[TODO]
 	public void DrawImage(Image image, Rectangle destRect,
 						  float srcX, float srcY,
 						  float srcWidth, float srcHeight,
 						  GraphicsUnit srcUnit, ImageAttributes imageAttr,
 						  DrawImageAbort callback, IntPtr callbackData)
 			{
-				// TODO
-				DrawImage(image, destRect, srcX, srcY, srcWidth, srcHeight, srcUnit);
+				DrawImage(image, destRect, srcX, srcY, srcWidth,
+						  srcHeight, srcUnit);
 			}
 
 	// Draw an unscaled image.
@@ -1095,7 +1135,6 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 			{
 				DrawImageUnscaled(image, point.X, point.Y);
 			}
-
 	public void DrawImageUnscaled(Image image, Rectangle rect)
 			{
 				DrawImageUnscaled(image, rect.X, rect.Y);
@@ -1348,9 +1387,10 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					int charactersFitted, linesFilled;
 					Size size = ToolkitGraphics.MeasureString
 						(s, null, null, out charactersFitted, out linesFilled, false);
+					bool containsNL =  (s.IndexOf('\n') >= 0);
 					// If we need to wrap then do it the hard way.
 					if((format.FormatFlags & StringFormatFlags.NoWrap) == 0 &&
-						size.Width >= deviceLayout.Width )
+						(size.Width >= deviceLayout.Width || containsNL) )
 					{
 						StringDrawPositionCalculator calculator = new StringDrawPositionCalculator(s, this, font, deviceLayout, format);
 						calculator.LayoutByWords();
@@ -1390,27 +1430,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	public void DrawString(String s, Font font, Brush brush,
 						   float x, float y, StringFormat format)
 			{
-				// TODO
-				if(s == null || s.Length == 0)
-				{
-					return;
-				}
-				int dx, dy;
-				ConvertPoint(x, y, out dx, out dy, pageUnit);
-				dx += baseWindow.X;
-				dy += baseWindow.Y;
-				// Simple optimization
-				if (clip != null)
-				{
-					if (dx > deviceClipExtent.Right || dy > deviceClipExtent.Bottom)
-						return;
-				}
-				lock(this)
-				{
-					SelectFont(font);
-					SelectBrush(brush);
-					ToolkitGraphics.DrawString(s, dx, dy, format);
-				}
+				DrawString(s, font, brush, new RectangleF(x, y, 999999.0f, 999999.0f), format);
 			}
 
 	// Reset the graphics state back to a previous container level.
@@ -2073,6 +2093,8 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				return Clip.IsVisible(x, y, width, height, this);
 			}
 
+	// First attempt as a way to measure strings and draw strings, taking into account a string format.
+	// The string measuring happens a word at a time.
 	private class StringDrawPositionCalculator
 	{
 		private SplitWord[] words;
@@ -2111,6 +2133,8 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 
 		public void LayoutByWords()
 				{
+					// Break the string into "words". Each word has a start pos, end pos and measured size.
+					// Each new line group is treated as a "word" as is each whitespace.
 					wordCount = 0;
 					words = new SplitWord[16];
 					int start = 0;
@@ -2118,31 +2142,46 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					{
 						start = i;
 						char c = text[i];
+						// Look for \r on its own, \n on its own or \r\n.
 						if(c == '\r')
 						{
 							if (i < text.Length-1 && text[i+1]=='\n')
+							{
 								i += 2;
+							}
 							else
-								i++; // Invalid, the \r is on its own
+							{
+								i++;
+							}
+						}
+						else if(c == '\n')
+						{
+							i++;
 						}
-						else if(c == '\n') // Invalid, the \n is on its own
-							i += 1;
 						else
 						{
-							while(i < text.Length && Char.IsWhiteSpace(text[i])) 
+							// Skip over the whitespace.
+							while(i < text.Length && Char.IsWhiteSpace(text[i]))
+							{
 								i++;
+							}
+							// We are at the start of text so skip over the text.
 							if (i == start)
 							{
 								while(i < text.Length && !Char.IsWhiteSpace(text[i]) && text[i] != '\n' && text[i] != '\r')
+								{
 									i++;
-							}
-							if (wordCount >= words.Length)
-							{
-								SplitWord[] newWords = new SplitWord[words.Length * 2];
-								Array.Copy(words, newWords, words.Length);
-								words = newWords;
+								}
 							}
 						}
+						// Dynamically allocate the array if we need more space.
+						if (wordCount >= words.Length)
+						{
+							SplitWord[] newWords = new SplitWord[words.Length * 2];
+							Array.Copy(words, newWords, words.Length);
+							words = newWords;
+						}
+						// Add the word.
 						words[wordCount++] = new SplitWord(start, i - start);
 					}	
 					Layout();
@@ -2192,6 +2231,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					}
 				}
 
+		// Calculate the bounds of the measured strings, the number of characters fitted and the number of lines.
 		public Size GetBounds(out int charactersFitted, out int linesFilled)
 				{
 					linesFilled = 0;
@@ -2202,56 +2242,51 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					}
 					bool noPartialLines = (format.FormatFlags & StringFormatFlags.LineLimit) != 0;
 					int h = words[0].size.Height;
+					// Find number of lines filled.
 					for (int i = 0; i < linePositions.Length; i++)
 					{
 						if (noPartialLines)
 						{
 							if (linePositions[i].Y + h > layout.Height)
+							{
 								break;
+							}
 						}
 						else if (linePositions[i].Y > layout.Height)
+						{
 							break;
+						}
 						linesFilled++;
 					}
-					if (linesFilled == 1)
+
+					int maxWidth = 0;
+					int currentWidth = 0;
+					int currentLine = 0;
+					// Find the maximum width of a line and the number of characters fitted.
+					for(int i=0; i < wordCount; i++)
 					{
-						// Find the width of the single line.
-						int width = 0;
-						for (int i = 0; i < wordCount; i++)
+						SplitWord word = words[i];
+						charactersFitted += word.length;
+						if(word.line == currentLine)
 						{
-							SplitWord word = words[i];
-							if (word.line != -1)
-								break;
-							width += word.size.Width;
-							charactersFitted += word.length;
+							currentWidth += word.size.Width;
 						}
-						return new Size(width, h);
-					}
-					else
-					{
-						// Find the characters fitted.
-						int width = (int)layout.Width;
-						for (int i = 0; i < wordCount; i++)
+						else
 						{
-							SplitWord word = words[i];
-							if (word.line != -1)
-								continue;
-							// if we are on the last line - check the width.
-							if (i == linesFilled - 1)
+							if (currentWidth > maxWidth)
 							{
-								width -= word.size.Width;
-								if (width < 0)
-								{
-									// TODO: This is a guess and might be inaccurate!!
-									// We should measure each character.
-									charactersFitted += ((-width * word.length + word.size.Width/2) / word.size.Width);
-									break;
-								}
+								maxWidth = currentWidth;
 							}
-							charactersFitted += word.length;
+							currentWidth = word.size.Width;
+							currentLine++;
 						}
-						return new Size((int)layout.Width, h * linesFilled);
 					}
+					if (currentWidth > maxWidth)
+					{
+						maxWidth = currentWidth;
+					}
+
+					return new Size(maxWidth, h * linesFilled);
 				}
 		// Use the toolkit to measure all the words and spaces.
 		private void  MeasureStrings() 
@@ -2265,17 +2300,22 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					{
 						SplitWord word = words[i];
 						char c = text[word.start];
-						if (char.IsWhiteSpace(c))
+						if (c != '\n' && c != '\r')
 						{
-							if (spaceSize.Width == -1)
-								spaceSize = graphics.ToolkitGraphics.MeasureString(" ", rect, null, out charactersFitted, out linesFilled, false);
-							word.size = spaceSize;
-							words[i] = word;
-						}
-						else if (c != '\n' && c != '\r')
-						{
-							word.size = graphics.ToolkitGraphics.MeasureString(text.Substring(word.start, word.length), rect, null, out charactersFitted, out linesFilled, false);
-							words[i] = word;
+							if (char.IsWhiteSpace(c))
+							{
+								if (spaceSize.Width == -1)
+								{
+									spaceSize = graphics.ToolkitGraphics.MeasureString(" ", rect, null, out charactersFitted, out linesFilled, false);
+								}
+								word.size = spaceSize;
+								words[i] = word;
+							}
+							else
+							{
+								word.size = graphics.ToolkitGraphics.MeasureString(text.Substring(word.start, word.length), rect, null, out charactersFitted, out linesFilled, false);
+								words[i] = word;
+							}
 						}
 					}
 				}
@@ -2291,10 +2331,11 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 						SplitWord word = words[i];
 						char c = text[word.start];
 						// Wrap when \r\n
-						if (c == '\r' && word.length == 2) 
+						if (c == '\r' || c == '\n') 
 						{
 							currLine++;
 							currSize = 0;
+							continue;
 						}
 						if (Char.IsWhiteSpace(c))
 						{
@@ -2302,7 +2343,8 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 							{
 								// Check that we are not at the end of the line.
 								SplitWord nextWord = words[i+1];
-								if (text[nextWord.start] != '\r' || nextWord.length != 2)
+								char c1 = text[nextWord.start];
+								if (c1 != '\r' && c1 != '\n')
 								{
 									// If we have space for the next word in the line then set the lines.
 									if (currSize + word.size.Width + nextWord.size.Width <= lineSize) 
@@ -2360,9 +2402,13 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					int yOffset=0;
 					// Set the offset based on the  LineAlignment.
 					if (format.LineAlignment == StringAlignment.Far)
+					{
 						yOffset = (int)layout.Height - numLines * h;
+					}
 					else if (format.LineAlignment == StringAlignment.Center)
+					{
 						yOffset = ((int)layout.Height - numLines * h)/2;
+					}
 
 					for(int line=0; line < numLines; line++)
 					{
@@ -2373,15 +2419,23 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 							{
 								SplitWord word = words[i];
 								if(word.line == line)
+								{
 									xOffset += word.size.Width;
+								}
 								if(word.line > line)
+								{
 									break;
+								}
 							}
 							// Set the offset based on the Alignment.
 							if (format.Alignment == StringAlignment.Far)
+							{
 								xOffset = (int)layout.Width - 1 - xOffset;
+							}
 							else if (format.Alignment == StringAlignment.Center)
+							{
 								xOffset = ((int)layout.Width - 1 - xOffset)/2;
+							}
 						}
 
 						linePositions[line].Y=(int)layout.Y + yOffset + line*h;
@@ -2657,11 +2711,13 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				SelectFont(font);
 				Size size = ToolkitGraphics.MeasureString
 					(text, null, null, out charactersFitted, out linesFilled, false);
+				bool containsNL =  (text.IndexOf('\n') >= 0);
+					
 				// If we need to wrap then do it the hard way.
 				if((format.FormatFlags & StringFormatFlags.NoWrap) == 0 &&
-					size.Width >= layoutArea.Width && layoutArea.Width != 0 )
+					(size.Width >= layoutArea.Width && layoutArea.Width != 0 || containsNL) )
 				{
-					Rectangle layout = new Rectangle(0,0, (int)layoutArea.Width - 1, (int)layoutArea.Height - 1);
+					Rectangle layout = new Rectangle(0,0, (int)layoutArea.Width, (int)layoutArea.Height);
 					StringDrawPositionCalculator calculator = new StringDrawPositionCalculator(text, this, font, layout , format);
 					calculator.LayoutByWords();
 					return calculator.GetBounds(out charactersFitted, out linesFilled);
@@ -3035,6 +3091,11 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 				UpdateClip();
 			}
 
+	internal void SetClipInternal(Region region)
+	{
+		clip = region;
+	}
+
 	// Transform points from one co-ordinate space to another.
 	public void TransformPoints(CoordinateSpace destSpace,
 		CoordinateSpace srcSpace,
@@ -3397,6 +3458,7 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 					Point p = points[i];
 					p.X += baseWindow.X;
 					p.Y += baseWindow.Y;
+					points[i] = p;
 				}
 			}
 	
@@ -3612,19 +3674,37 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	private Point[] ConvertRectangle3(int x, int y, int width, int height, GraphicsUnit unit)
 			{
 				Point[] points = new Point[3];
-				points[0] = new Point(x, y);
-				points[1] = new Point(x + width, y);
-				points[2] = new Point(x, y + height);
-				return ConvertPoints(points, 3, unit);
+				int pt1x, pt1y, pt2x, pt2y, pt3x, pt3y;
+				pt1x = x;
+				pt1y = y;
+				pt2x = x + width;
+				pt2y = y;
+				pt3x = x;
+				pt3y = y + height;
+				if(transform != null || pageScale != 1.0f ||
+				   (unit != GraphicsUnit.Pixel && unit != GraphicsUnit.World))
+				{
+					ConvertPoint(ref pt1x, ref pt1y, unit);
+					ConvertPoint(ref pt2x, ref pt2y, unit);
+					ConvertPoint(ref pt3x, ref pt3y, unit);
+				}
+				points[0] = new Point(pt1x, pt1y);
+				points[1] = new Point(pt2x, pt2y);
+				points[2] = new Point(pt3x, pt3y);
+				return points;
 			}
 
 	private Point[] ConvertRectangle3(float x, float y, float width, float height, GraphicsUnit unit)
 			{
-				PointF[] points = new PointF[3];
-				points[0] = new PointF(x, y);
-				points[1] = new PointF(x + width, y);
-				points[3] = new PointF(x, y + height);
-				return ConvertPoints(points, 3, unit);
+				Point[] points = new Point[3];
+				int pt1x, pt1y, pt2x, pt2y, pt3x, pt3y;
+				ConvertPoint(x, y, out pt1x, out pt1y, unit);
+				ConvertPoint(x + width, y, out pt2x, out pt2y, unit);
+				ConvertPoint(x, y + height, out pt3x, out pt3y, unit);
+				points[0] = new Point(pt1x, pt1y);
+				points[1] = new Point(pt2x, pt2y);
+				points[2] = new Point(pt3x, pt3y);
+				return points;
 			}
 
 	// Convert a rectangle into a set of 4 device co-ordinates.
@@ -3820,7 +3900,15 @@ public sealed class Graphics : MarshalByRefObject, IDisposable
 	// Update the clipping region within the IToolkitGraphics object.
 	private void UpdateClip()
 			{
-				RectangleF[] rectsF = clip.GetRegionScans(Transform);
+				RectangleF[] rectsF;
+				if (transform == null && pageScale == 1.0f && pageUnit == GraphicsUnit.World)
+				{
+					rectsF = clip.GetRegionScansIdentity();
+				}
+				else
+				{
+					rectsF = clip.GetRegionScans(Transform);
+				}
 				int left = int.MaxValue;
 				int top = int.MaxValue;
 				int right = int.MinValue;
