@@ -98,6 +98,7 @@ namespace System.Windows.Forms
 			moveButtonLeftState = ButtonState.Normal;
 			moveButtonRightState = ButtonState.Normal;
 			hotTrackIndex = -1;
+			SetStyle(ControlStyles.DoubleBuffer | ControlStyles.AllPaintingInWmPaint | ControlStyles.UserPaint, true);
 		}
 
 		public Control GetControl(int index)
@@ -418,6 +419,11 @@ namespace System.Windows.Forms
 				return tabOwner.Controls.Contains(control);
 			}
 
+			public bool Contains(TabPage control)
+			{
+				return tabOwner.Controls.Contains(control);
+			}
+
 			public int IndexOf(TabPage control)
 			{
 				return tabOwner.Controls.IndexOf(control);
