@@ -70,7 +70,8 @@ public class TextBox : TextBoxBase
 		// Trap interesting events.  We do it this way rather
 		// than override virtual methods so that the published
 		// TextBox API is maintained.
-		KeyPress += new KeyPressEventHandler(HandleKeyPress);
+		// Note: except for KeyPress events, for which the 
+		// hooked up calls get priority over TextBox class.
 		MouseDown += new MouseEventHandler(HandleMouseDown);
 		MouseMove += new MouseEventHandler(HandleMouseMove);
 		DoubleClick += new EventHandler(HandleDoubleClick);
@@ -541,10 +542,20 @@ public class TextBox : TextBoxBase
 		}
 	}
 
+	protected override void OnKeyPress(KeyPressEventArgs e)
+	{
+		base.OnKeyPress(e);
+		
+		if(e.Handled == false)
+		{
+			HandleKeyPress(this, e);
+		}
+	}
+
 	// Handle "KeyPress" events for the text box.
 	private void HandleKeyPress(Object sender, KeyPressEventArgs e)
 	{
-		if (ReadOnly)
+		if (ReadOnly || e.Handled)
 		{
 			return;
 		}
@@ -1674,7 +1685,7 @@ public class TextBox : TextBoxBase
 
 	private void SelectWord(int pos)
 	{
-		if (pos >= text.Length)
+		if (text == null || pos >= text.Length)
 		{
 			return;
 		}
