@@ -1,7 +1,8 @@
 /*
  * XmlDocument.cs - Implementation of the "System.Xml.XmlDocument" class.
  *
- * Copyright (C) 2002 Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2002  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2004  Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,7 @@ using System;
 using System.IO;
 using System.Text;
 using System.Xml.Private;
+using System.Xml.XPath;
 
 #if ECMA_COMPAT
 internal
@@ -215,7 +217,7 @@ class XmlDocument : XmlNode
 			}
 
 	// Set the resolver to use for external resources.
-	public XmlResolver XmlResolver
+	public virtual XmlResolver XmlResolver
 			{
 				set
 				{
@@ -298,6 +300,16 @@ class XmlDocument : XmlNode
 				return new XmlComment(placeholder, text);
 			}
 
+	// Create a default attribute and associate it with this document.
+	protected internal virtual XmlAttribute CreateDefaultAttribute
+				(String prefix, String localName, String namespaceURI)
+			{
+				XmlAttribute att = CreateAttribute
+					(prefix, localName, namespaceURI);
+				att.isDefault = true;
+				return att;
+			}
+
 	// Create a document fragment that is attached to this node.
 	public virtual XmlDocumentFragment CreateDocumentFragment()
 			{
@@ -314,7 +326,7 @@ class XmlDocument : XmlNode
 			}
 
 	// Create an element that is attached to this node.
-	public virtual XmlElement CreateElement(String name)
+	public XmlElement CreateElement(String name)
 			{
 				int colon = name.LastIndexOf(':');
 				if(colon == -1)
@@ -335,8 +347,7 @@ class XmlDocument : XmlNode
 					}
 				}
 			}
-	public virtual XmlElement CreateElement(String qualifiedName,
-											String namespaceURI)
+	public XmlElement CreateElement(String qualifiedName, String namespaceURI)
 			{
 				int colon = qualifiedName.LastIndexOf(':');
 				if(colon == -1)
@@ -588,6 +599,13 @@ class XmlDocument : XmlNode
 				return new XmlDeclaration
 					(placeholder, version, encoding, standalone);
 			}
+	
+	// NOTE: should be protected internal virtual 
+	// but for ml-pnet's System.Data the internal modifier has to be removed
+	protected virtual XPathNavigator CreateNavigator(XmlNode node)
+			{
+				return node.CreateNavigator();
+			}
 
 	// Get an element by identifier.
 	[TODO]
@@ -598,20 +616,16 @@ class XmlDocument : XmlNode
 			}
 
 	// Get a list of elements by tag name.
-	[TODO]
 	public virtual XmlNodeList GetElementsByTagName(String name)
 			{
-				// TODO
-				return null;
+				return DocumentElement.GetElementsByTagName(name);
 			}
 
 	// Get a list of elements by local name and namespace.
-	[TODO]
 	public virtual XmlNodeList GetElementsByTagName
 				(String name, String namespaceURI)
 			{
-				// TODO
-				return null;
+				return DocumentElement.GetElementsByTagName(name, namespaceURI);
 			}
 
 	// Import a node from another document.
@@ -680,7 +694,7 @@ class XmlDocument : XmlNode
 			}
 
 	// Read the children of a node.
-	private void ReadChildren(XmlReader r, XmlNode parent)
+	internal void ReadChildren(XmlReader r, XmlNode parent)
 			{
 				if(parent is XmlAttribute)
 				{
@@ -739,8 +753,8 @@ class XmlDocument : XmlNode
 					case XmlNodeType.Attribute:
 					{
 						// create the attribute
-						XmlAttribute att = CreateAttribute(r.Name);
-						att.Value = r.Value;
+						XmlAttribute att = CreateAttribute
+							(r.Prefix, r.LocalName, r.NamespaceURI);
 
 						// read and append the children
 						ReadChildren(r, att);
@@ -806,7 +820,14 @@ class XmlDocument : XmlNode
 						r.MoveToElement();
 
 						// return now if there are no children
-						if(isEmptyElement) { return elem; }
+						if(isEmptyElement)
+						{
+							// advance the reader to the next node
+							r.Read();
+
+							// return the empty element
+							return elem;
+						}
 
 						// read and append the children
 						ReadChildren(r, elem);
@@ -1093,26 +1114,112 @@ class XmlDocument : XmlNode
 				return result;
 			}
 
-	// Save XML data from this document.
-	[TODO]
+	// Save the contents of this document to a stream.
 	public virtual void Save(Stream outStream)
 			{
-				// TODO
+				// check for a root element
+				if(DocumentElement == null)
+				{
+					throw new XmlException
+						(S._("Xml_NoRootElement"));
+				}
+
+				// get the xml declaration
+				XmlDeclaration dec = (XmlDeclaration)GetChildByType
+					(XmlNodeType.XmlDeclaration);
+
+				// determine the encoding
+				Encoding enc = null;
+				if(dec != null)
+				{
+					String encName = dec.Encoding;
+					if(encName != null && encName.Length != 0)
+					{
+						enc = Encoding.GetEncoding(encName);
+					}
+				}
+
+				// create the writer
+				XmlTextWriter w = new XmlTextWriter(outStream, enc);
+
+				// turn on formatting if whitespace preservation is off
+				if(!preserveWhitespace) { w.Formatting = Formatting.Indented; }
+
+				// make sure we have an xml declaration
+				if(dec == null) { w.WriteStartDocument(); }
+
+				// write this document to the writer
+				WriteTo(w);
+
+				// flush the output
+				w.Flush();
 			}
-	[TODO]
+
+	// Save the contents of this document to a file.
 	public virtual void Save(String filename)
 			{
-				// TODO
+				// create a stream for the file
+				FileStream fs = new FileStream
+					(filename, FileMode.Create, FileAccess.Write);
+
+				// save using the stream version of save
+				Save(fs);
+
+				// close the file stream
+				fs.Close();
 			}
-	[TODO]
+
+	// Save the contents of this document to a writer.
 	public virtual void Save(TextWriter writer)
 			{
-				// TODO
+				// create a new xml text writer using the given writer
+				XmlTextWriter w = new XmlTextWriter(writer);
+
+				// turn on formatting if whitespace preservation is off
+				if(!preserveWhitespace) { w.Formatting = Formatting.Indented; }
+
+				// get the xml declaration
+				XmlDeclaration dec = (XmlDeclaration)GetChildByType
+					(XmlNodeType.XmlDeclaration);
+
+				// reset the declaration's encoding to that of the writer
+				if(dec != null)
+				{
+					Encoding enc = writer.Encoding;
+					String encName = String.Empty;
+					if(enc != null) { encName = enc.WebName; }
+					dec.Encoding = encName;
+				}
+
+				// save using the xml text writer version of save
+				Save(w);
 			}
-	[TODO]
+
+	// Save the contents of this document to an XML writer.
 	public virtual void Save(XmlWriter w)
 			{
-				// TODO
+				// check for a root element
+				if(DocumentElement == null)
+				{
+					throw new XmlException
+						(S._("Xml_NoRootElement"));
+				}
+
+				// get the xml declaration
+				XmlDeclaration dec = (XmlDeclaration)GetChildByType
+					(XmlNodeType.XmlDeclaration);
+
+				// handle the encoding and xml declaration stuff
+				if(dec == null)
+				{
+					w.WriteStartDocument();
+				}
+
+				// write this document to the writer
+				WriteTo(w);
+
+				// flush the output
+				w.Flush();
 			}
 
 	// Write the contents of this document to an XML writer.
