@@ -65,7 +65,7 @@ class XmlElement : XmlLinkedNode
 			}
 
 	// Determine if this element node has attributes.
-	public bool HasAttributes
+	public virtual bool HasAttributes
 			{
 				get
 				{
@@ -281,6 +281,7 @@ class XmlElement : XmlLinkedNode
 	// Get a list of all descendents that match a particular name.
 	public virtual XmlNodeList GetElementsByTagName(String name)
 			{
+				name = (FindOwnerQuick()).NameTable.Add(name);
 				return new ElementList(this, name);
 			}
 
@@ -288,6 +289,9 @@ class XmlElement : XmlLinkedNode
 	public virtual XmlNodeList GetElementsByTagName
 				(String localName, String namespaceURI)
 			{
+				XmlNameTable nt = FindOwnerQuick().NameTable;
+				localName = nt.Add(localName);
+				namespaceURI = nt.Add(namespaceURI);
 				return new ElementList(this, localName, namespaceURI);
 			}
 
@@ -401,8 +405,8 @@ class XmlElement : XmlLinkedNode
 			}
 
 	// Set an attribute to a specific value.
-	public virtual void SetAttribute(String localName, String namespaceURI,
-									 String value)
+	public virtual String SetAttribute
+				(String localName, String namespaceURI, String value)
 			{
 				XmlAttribute attr = GetAttributeNode(localName, namespaceURI);
 				if(attr != null)
@@ -416,6 +420,7 @@ class XmlElement : XmlLinkedNode
 					attr.Value = value;
 					Attributes.Append(attr);
 				}
+				return attr.Value;
 			}
 
 	// Set an attribute by node.
