@@ -1970,7 +1970,7 @@ public class XmlTextReader : XmlReader
 				if(namespaces)
 				{
 					attributes.UpdateInfo
-						(context.NameTable, context.NamespaceManager, namespaceURI);
+						(context.NameTable, context.NamespaceManager);
 				}
 				else
 				{
@@ -1986,6 +1986,10 @@ public class XmlTextReader : XmlReader
 	//
 	// Already read: ''
 	private void ReadText()
+			{
+				ReadText(false);
+			}
+	private void ReadText(bool fromWhitespace)
 			{
 				// check the state
 				CheckState(State.Content);
@@ -1997,9 +2001,19 @@ public class XmlTextReader : XmlReader
 				Segments segments = info.Segments;
 				XmlNameTable nt = context.NameTable;
 
-				// create our log and push it onto the logger's log stack
-				StringBuilder log = new StringBuilder();
-				input.Logger.Push(log);
+				// handle log setup, based on if text was read as whitespace
+				StringBuilder log;
+				if(fromWhitespace)
+				{
+					// copy the whitespace log from the logger's log stack
+					log = input.Logger.Peek();
+				}
+				else
+				{
+					// create a new log and push it onto the logger's log stack
+					log = new StringBuilder();
+					input.Logger.Push(log);
+				}
 
 				// read until we consume all the text
 				while(input.PeekChar() && input.peekChar != '<')
@@ -2077,6 +2091,13 @@ public class XmlTextReader : XmlReader
 				// skip the required whitespace
 				if(!input.SkipWhitespace()) { Error(/* TODO */); }
 
+				// check to see if we should treat the whitespace as text
+				if(input.PeekChar() && input.peekChar != '<')
+				{
+					ReadText(true);
+					return;
+				}
+
 				// get the whitespace from the log and pop it from the logger
 				String value = input.Logger.Pop().ToString();
 
