@@ -408,9 +408,12 @@ public sealed class TypeDescriptor
 					{
 						Type converterType =
 							Type.GetType(attr.ConverterTypeName);
-						element.converter = (TypeConverter)
-							Activator.CreateInstance(converterType);
-						return element.converter;
+						if(converterType != null)
+						{
+							element.converter = (TypeConverter)
+								Activator.CreateInstance(converterType);
+							return element.converter;
+						}
 					}
 					element.converter = GetBuiltinConverter(type);
 					return element.converter;
@@ -452,7 +455,7 @@ public sealed class TypeDescriptor
 					DefaultEventAttribute attr = (DefaultEventAttribute)
 						GetAttributeForType(componentType,
 											typeof(DefaultEventAttribute));
-					if(attr != null)
+					if(attr != null && attr.Name != null)
 					{
 						element.defaultEvent =
 							CreateEvent(componentType, attr.Name, null, null);
@@ -501,7 +504,7 @@ public sealed class TypeDescriptor
 					DefaultPropertyAttribute attr = (DefaultPropertyAttribute)
 						GetAttributeForType(componentType,
 											typeof(DefaultPropertyAttribute));
-					if(attr != null)
+					if(attr != null && attr.Name != null)
 					{
 						element.defaultProperty =
 							CreateProperty
