@@ -1,7 +1,7 @@
 /*
  * Graphics.cs - Graphic drawing objects.
  *
- * Copyright (C) 2002, 2003  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2002, 2003, 2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ namespace Xsharp
 
 using System;
 using Xsharp.Types;
+using OpenSystem.Platform.X11;
 
 /// <summary>
 /// <para>The <see cref="T:Xsharp.Graphics"/> class manages a graphic
@@ -58,7 +59,7 @@ public sealed class Graphics : IDisposable
 	// Internal state.
 	internal Display dpy;
 	private Drawable drawable;
-	internal Xlib.Drawable drawableHandle;
+	internal XDrawable drawableHandle;
 	internal IntPtr gc;
 	private Color foreground;
 	private Color background;
@@ -108,12 +109,18 @@ public sealed class Graphics : IDisposable
 				this.drawable = drawable;
 				XGCValues gcValues = new XGCValues();
 				InputOutputWidget widget = (drawable as InputOutputWidget);
+				DoubleBuffer buffer = (drawable as DoubleBuffer);
 				Bitmap bitmap = (drawable as Bitmap);
 				if(widget != null)
 				{
 					foreground = widget.Foreground;
 					background = widget.Background;
 				}
+				else if(buffer != null)
+				{
+					foreground = buffer.Widget.Foreground;
+					background = buffer.Widget.Background;
+				}
 				else if(bitmap != null)
 				{
 					foreground = new Color(0x00, 0x00, 0x00);
@@ -126,6 +133,10 @@ public sealed class Graphics : IDisposable
 				}
 				gcValues.foreground = drawable.ToPixel(foreground);
 				gcValues.background = drawable.ToPixel(background);
+				if(drawable is DoubleBuffer)
+				{
+					((DoubleBuffer)drawable).Start(this);
+				}
 				try
 				{
 					IntPtr display = dpy.Lock();
@@ -149,7 +160,7 @@ public sealed class Graphics : IDisposable
 						// Xlib will take care of stripping the list down
 						// to just the changes that need to be applied.
 						gcValues.function = Xsharp.Function.GXcopy;
-						gcValues.plane_mask = ~((Xlib.Pixel)0);
+						gcValues.plane_mask = ~((XPixel)0);
 						gcValues.line_width = 0;
 						gcValues.line_style = Xsharp.LineStyle.LineSolid;
 						gcValues.cap_style = Xsharp.CapStyle.CapButt;
@@ -164,7 +175,7 @@ public sealed class Graphics : IDisposable
 						gcValues.graphics_exposures = true;
 						gcValues.clip_x_origin = 0;
 						gcValues.clip_y_origin = 0;
-						gcValues.clip_mask = Xlib.Pixmap.Zero;
+						gcValues.clip_mask = XPixmap.Zero;
 						gcValues.dash_offset = 0;
 						gcValues.dashes = (sbyte)4;
 						Xlib.XChangeGC(display, gc,
@@ -195,6 +206,10 @@ public sealed class Graphics : IDisposable
 				{
 					dpy.Unlock();
 				}
+				if(drawable is DoubleBuffer)
+				{
+					((DoubleBuffer)drawable).ClearAtStart(this);
+				}
 			}
 
 	/// <summary>
@@ -212,6 +227,12 @@ public sealed class Graphics : IDisposable
 					IntPtr display = dpy.Lock();
 					if(gc != IntPtr.Zero)
 					{
+						// Flush the double buffer if necessary.
+						if(drawable is DoubleBuffer)
+						{
+							((DoubleBuffer)drawable).End(this);
+						}
+
 						// Release the GC back to the screen's cache so
 						// that we can reuse it the next time we need a GC.
 						drawable.screen.ReleaseGC(gc, (drawable is Bitmap));
@@ -234,7 +255,7 @@ public sealed class Graphics : IDisposable
 	private IntPtr Lock()
 			{
 				IntPtr display = dpy.Lock();
-				if(drawable.handle != Xlib.Drawable.Zero && gc != IntPtr.Zero)
+				if(drawable.handle != XDrawable.Zero && gc != IntPtr.Zero)
 				{
 					// All of the relevant handles are still valid.
 					return display;
@@ -1157,7 +1178,7 @@ public sealed class Graphics : IDisposable
 					}
 					else
 					{
-						Xlib.XSetClipMask(display, gc, Xlib.Pixmap.Zero);
+						Xlib.XSetClipMask(display, gc, XPixmap.Zero);
 					}
 				}
 				finally
@@ -1209,7 +1230,7 @@ public sealed class Graphics : IDisposable
 					}
 					else
 					{
-						Xlib.XSetClipMask(display, gc, Xlib.Pixmap.Zero);
+						Xlib.XSetClipMask(display, gc, XPixmap.Zero);
 					}
 				}
 				finally
@@ -1234,7 +1255,15 @@ public sealed class Graphics : IDisposable
 				}
 
 				// Determine the background pattern to clear with.
-				if((widget = (drawable as InputOutputWidget)) != null)
+				if(drawable is DoubleBuffer)
+				{
+					widget = ((DoubleBuffer)drawable).Widget;
+				}
+				else
+				{
+					widget = (drawable as InputOutputWidget);
+				}
+				if(widget != null)
 				{
 					widget.GetBackgroundInfo(out bg, out pixmap,
 											 out tx, out ty);
@@ -2164,7 +2193,8 @@ public sealed class Graphics : IDisposable
 				}
 
 				// Can we take a short-cut using the XImage?
-				if(image.ShouldUseXImage)
+				if(image.ShouldUseXImage && !(drawable is Pixmap) &&
+				   !(drawable is DoubleBuffer))
 				{
 					// Use "PutXImage" to draw through the clip mask,
 					// to avoid having to create a Pixmap in the server.
@@ -2260,7 +2290,8 @@ public sealed class Graphics : IDisposable
 				}
 
 				// Can we take a short-cut using the XImage?
-				if(image.ShouldUseXImage)
+				if(image.ShouldUseXImage && !(drawable is Pixmap) &&
+				   !(drawable is DoubleBuffer))
 				{
 					// Use "PutXImage" to draw through the clip mask,
 					// to avoid having to create a Pixmap in the server.
@@ -2282,6 +2313,127 @@ public sealed class Graphics : IDisposable
 				SetClipMask(null);
 			}
 
+	/// <summary>
+	/// <para>Blit a drawable sub-part to this graphics context.</para>
+	/// </summary>
+	///
+	/// <param name="x">
+	/// <para>The X co-ordinate of the top-left point of
+	/// the destination bounding rectangle.</para>
+	/// </param>
+	///
+	/// <param name="y">
+	/// <para>The Y co-ordinate of the top-left point of
+	/// the destination bounding rectangle.</para>
+	/// </param>
+	///
+	/// <param name="drawable">
+	/// <para>The drawable to copy from.</para>
+	/// </param>
+	///
+	/// <param name="srcX">
+	/// <para>The X co-ordinate of the top-left point of
+	/// the drawable sub-part to be drawn.</para>
+	/// </param>
+	///
+	/// <param name="srcY">
+	/// <para>The Y co-ordinate of the top-left point of
+	/// the drawable sub-part to be drawn.</para>
+	/// </param>
+	///
+	/// <param name="srcWidth">
+	/// <para>The width of the drawable sub-part to be drawn.</para>
+	/// </param>
+	///
+	/// <param name="srcHeight">
+	/// <para>The height of the drawable sub-part to be drawn.</para>
+	/// </param>
+	///
+	/// <exception cref="T:System.ArgumentNullException">
+	/// <para>Raised if <paramref name="drawable"/> is <see langword="null"/>.
+	/// </para>
+	/// </exception>
+	///
+	/// <exception cref="T:Xsharp.XException">
+	/// <para>One of the co-ordinate or size values is out of range.</para>
+	/// </exception>
+	public void BitBlt(int x, int y, Drawable drawable,
+					   int srcX, int srcY, int srcWidth, int srcHeight)
+			{
+				// Validate the drawable parameter.
+				if(drawable == null)
+				{
+					throw new ArgumentNullException("drawable");
+				}
+
+				// Bail out if the source co-ordinates are out of range.
+				int width, height;
+				width = drawable.Width;
+				height = drawable.Height;
+				if(srcX < 0 || srcX >= width ||
+				   srcY < 0 || srcY >= height ||
+				   srcWidth <= 0 || srcHeight <= 0 ||
+				   srcWidth > width || srcHeight > height ||
+				   srcX > (width - srcWidth) ||
+				   srcY > (height - srcHeight))
+				{
+					throw new XException(S._("X_RectCoordRange"));
+				}
+
+				// Blit the contents of the drawable.
+				try
+				{
+					IntPtr display = Lock();
+					Xlib.XCopyArea(display, drawable.GetGCHandle(),
+								   drawableHandle, gc,
+								   srcX, srcY,
+								   (uint)srcWidth, (uint)srcHeight,
+								   x, y);
+				}
+				finally
+				{
+					dpy.Unlock();
+				}
+			}
+
+	/// <summary>
+	/// <para>Blit a drawable's entire contents to this graphics context.</para>
+	/// </summary>
+	///
+	/// <param name="x">
+	/// <para>The X co-ordinate of the top-left point of
+	/// the destination bounding rectangle.</para>
+	/// </param>
+	///
+	/// <param name="y">
+	/// <para>The Y co-ordinate of the top-left point of
+	/// the destination bounding rectangle.</para>
+	/// </param>
+	///
+	/// <param name="drawable">
+	/// <para>The drawable to copy from.</para>
+	/// </param>
+	///
+	/// <exception cref="T:System.ArgumentNullException">
+	/// <para>Raised if <paramref name="drawable"/> is <see langword="null"/>.
+	/// </para>
+	/// </exception>
+	///
+	/// <exception cref="T:Xsharp.XException">
+	/// <para>One of the co-ordinate or size values is out of range.</para>
+	/// </exception>
+	public void BitBlt(int x, int y, Drawable drawable)
+			{
+				// Validate the drawable parameter.
+				if(drawable == null)
+				{
+					throw new ArgumentNullException("drawable");
+				}
+
+				// Perform the blit operation.
+				BitBlt(x, y, drawable, 0, 0, drawable.Width, drawable.Height);
+			}
+
 	/// <summary>
 	/// <para>Draw a string at a particular position using a
 	/// specified font.</para>
@@ -2401,12 +2553,12 @@ public sealed class Graphics : IDisposable
 	// Draw a radio button.
 	private void DrawRadio(IntPtr display,
 						   int x, int y, int width, int height,
-						   Xlib.Pixel topShadow,
-						   Xlib.Pixel topShadowEnhanced,
-						   Xlib.Pixel bottomShadow,
-						   Xlib.Pixel bottomShadowEnhanced,
-						   Xlib.Pixel foreground,
-						   Xlib.Pixel background)
+						   XPixel topShadow,
+						   XPixel topShadowEnhanced,
+						   XPixel bottomShadow,
+						   XPixel bottomShadowEnhanced,
+						   XPixel foreground,
+						   XPixel background)
 			{
 				// Adjust the top-left position of the radio button.
 				x += (width - BuiltinBitmaps.RadioWidth) / 2;
@@ -2457,7 +2609,7 @@ public sealed class Graphics : IDisposable
 
 	// Draw a simple bitmap.
 	internal void DrawBitmap(int x, int y, int width, int height,
-							 Xlib.Pixmap bitmap)
+							 XPixmap bitmap)
 			{
 				try
 				{
@@ -2523,12 +2675,12 @@ public sealed class Graphics : IDisposable
 					IntPtr display = Lock();
 
 					// Get the colors that we need to draw the effect.
-					Xlib.Pixel topShadow;
-					Xlib.Pixel topShadowEnhance;
-					Xlib.Pixel bottomShadow;
-					Xlib.Pixel bottomShadowEnhance;
-					Xlib.Pixel background;
-					Xlib.Pixel trim;
+					XPixel topShadow;
+					XPixel topShadowEnhance;
+					XPixel bottomShadow;
+					XPixel bottomShadowEnhance;
+					XPixel background;
+					XPixel trim;
 					if((effect & Effect.ContentColors) != 0)
 					{
 						topShadow = drawable.ToPixel
