@@ -333,7 +333,9 @@ public class Assembly : IClrProgramItem
 				}
 				else
 				{
-					assembly = LoadFromName(assemblyString, out error, caller);
+					AssemblyName name = AssemblyName.Parse(assemblyString);
+					assembly = LoadFromName(name.Name, out error, 
+											caller);
 				}
 				if(error == LoadError_OK)
 				{
@@ -358,13 +360,49 @@ public class Assembly : IClrProgramItem
 			}
 	internal static Assembly LoadFrom(String assemblyFile, Assembly caller)
 			{
+				char [] pathChars = new char[] {
+										Path.DirectorySeparatorChar,
+										Path.VolumeSeparatorChar,
+										Path.AltDirectorySeparatorChar };
+						
 				if(assemblyFile == null)
 				{
 					throw new ArgumentNullException("assemblyFile");
 				}
 				int error;
-				Assembly assembly = LoadFromFile(assemblyFile, out error,
+				Assembly assembly;
+
+				if(assemblyFile.Length >= 7 &&
+				   String.Compare(assemblyFile, 0, "file://", 0, 7, true)
+				   		== 0)
+				{
+					if(assemblyFile.Length >= 10 &&
+					   assemblyFile[7] == '/' &&
+					   assemblyFile[9] == ':')
+					{
+						// Specification of the form "file:///X:...".
+						assemblyFile = assemblyFile.Substring(8);
+					}
+					else
+					{
+						// Some other type of file specification.
+						assemblyFile = assemblyFile.Substring(7);
+					}
+				}
+				
+				if(assemblyFile.EndsWith(".dll") || 
+					assemblyFile.EndsWith(".DLL") ||
+					(assemblyFile.IndexOfAny(pathChars) != -1))
+				{
+					assembly = LoadFromFile(assemblyFile, out error,
 											     caller);
+				}
+				else
+				{
+					AssemblyName name = AssemblyName.Parse(assemblyFile);
+					assembly = LoadFromName(name.Name, out error,
+												 caller);
+				}
 				if(error == LoadError_OK)
 				{
 					return assembly;
@@ -461,12 +499,14 @@ public class Assembly : IClrProgramItem
 												Assembly caller)
 			{
 				Assembly assembly;
+				AssemblyName name;
 				int error;
 				if(partialName == null)
 				{
 					throw new ArgumentNullException("partialName");
 				}
-				assembly = LoadFromName(partialName, out error, caller);
+				name = AssemblyName.Parse(partialName);
+				assembly = LoadFromName(name.Name, out error, caller);
 				if(error == LoadError_OK)
 				{
 					return assembly;
