@@ -22,11 +22,12 @@
 namespace System.Runtime.Serialization.Formatters.Binary
 {
 
-#if CONFIG_SERIALIZATION
+//#if CONFIG_SERIALIZATION
 
 using System.IO;
 using System.Reflection;
 using System.Collections;
+using System.Runtime.Serialization;
 
 internal abstract class BinaryValueWriter
 {
@@ -49,6 +50,7 @@ internal abstract class BinaryValueWriter
 	private static BinaryValueWriter stringWriter = new StringWriter();
 	private static BinaryValueWriter objectWriter = new ObjectWriter();
 	private static BinaryValueWriter infoWriter = new SurrogateWriter(null);
+	private static BinaryValueWriter serializableWriter = new ISerializableWriter();
 	private static BinaryValueWriter arrayWriter = new ArrayWriter();
 
 	// Context information for writing binary values.
@@ -206,6 +208,7 @@ internal abstract class BinaryValueWriter
 
 				// Handle the primitive types first.
 				code = GetPrimitiveTypeCode(type);
+
 				switch(code)
 				{
 					case BinaryPrimitiveTypeCode.Boolean:
@@ -242,6 +245,12 @@ internal abstract class BinaryValueWriter
 						return stringWriter;
 				}
 
+				// Check for types that implement ISerializable.
+				if(typeof(ISerializable).IsAssignableFrom(type))
+				{
+					return serializableWriter;
+				}
+
 				// Handle special types that we recognize.
 				if(type == typeof(Object))
 				{
@@ -266,15 +275,9 @@ internal abstract class BinaryValueWriter
 					}
 				}
 
-				// Check for types that implement ISerializable.
-				if(typeof(ISerializable).IsAssignableFrom(type))
-				{
-					return infoWriter;
-				}
-
 				// Bail out if the type is not marked with the
 				// "serializable" flag.
-				if(!type.IsSerializable)
+				if(!type.IsSerializable && !type.IsInterface)
 				{
 					throw new SerializationException
 						(String.Format
@@ -290,6 +293,7 @@ internal abstract class BinaryValueWriter
 										  Assembly assembly)
 			{
 				String name = assembly.FullName;
+				
 				if(context.formatter.AssemblyFormat ==
 						FormatterAssemblyStyle.Full)
 				{
@@ -309,6 +313,23 @@ internal abstract class BinaryValueWriter
 				}
 			}
 
+	// if a member-name is used in the current class and in a base class, then the name has to be prefixed by <class>+
+	private static String GetMemberName(MemberInfo[] allMembers, MemberInfo member) {
+		bool prefix = false;
+		foreach(MemberInfo mi in allMembers) {
+			if(mi.Name == member.Name && mi.DeclaringType != member.DeclaringType) {
+				prefix = true;
+				break;
+			}
+		}
+
+		if(prefix) {
+			return member.DeclaringType.FullName+"+"+member.Name;
+		} else {
+			return member.Name;
+		}
+	}
+
 	// Write object values.
 	private class ObjectWriter : BinaryValueWriter
 	{
@@ -379,6 +400,10 @@ internal abstract class BinaryValueWriter
 							((byte)(BinaryElementType.NullValue));
 						return;
 					}
+					else if(type == typeof(String)) {
+						stringWriter.WriteInline(context, value, type, fieldType);
+						return;
+					}
 					else if(type.IsValueType)
 					{
 						if(fieldType.IsValueType)
@@ -455,11 +480,14 @@ internal abstract class BinaryValueWriter
 						MemberInfo[] members =
 							FormatterServices.GetSerializableMembers
 								(type, context.formatter.Context);
+						// write out number of members
+						context.writer.Write((int)members.Length);
+
 						int index;
 						Type fieldType;
 						for(index = 0; index < members.Length; ++index)
 						{
-							context.writer.Write(members[index].Name);
+							context.writer.Write(GetMemberName(members, members[index]));
 						}
 						for(index = 0; index < members.Length; ++index)
 						{
@@ -546,6 +574,117 @@ internal abstract class BinaryValueWriter
 
 	}; // class ObjectWriter
 
+	private class ISerializableWriter : ObjectWriter 
+	{
+		// Write the object header information for a type.
+		// IS11n objects are written like normal objects, but the
+		// members are replaced by the key->value pairs from the
+		// SerInfo.
+		public override void WriteObjectHeader(BinaryValueContext context,
+											   Object value, Type type,
+											   long objectID, long prevObject)
+				{
+					if(prevObject == -1)
+					{
+						// Write the full type information.
+						long assemblyID;
+						if(type.Assembly == Assembly.GetExecutingAssembly())
+						{
+							context.writer.Write
+								((byte)(BinaryElementType.RuntimeObject));
+							assemblyID = -1;
+						}
+						else
+						{
+							bool firstTime;
+							assemblyID = context.gen.GetId
+								(type.Assembly, out firstTime);
+							if(firstTime)
+							{
+								context.writer.Write
+									((byte)(BinaryElementType.Assembly));
+								context.writer.Write((int)assemblyID);
+								WriteAssemblyName(context, type.Assembly);
+							}
+							context.writer.Write
+								((byte)(BinaryElementType.ExternalObject));
+						}
+						context.writer.Write((int)objectID);
+						context.writer.Write(type.FullName);
+
+						// get members
+						StreamingContext streamContext = context.formatter.Context;
+						SerializationInfo info = new SerializationInfo(type, context.formatter.converter);
+						((ISerializable) value).GetObjectData(info, streamContext);
+
+						// write out number of members
+						context.writer.Write((int)info.MemberCount);
+
+						int index;
+						Type fieldType;
+
+						foreach(SerializationEntry entry in info)
+						{
+							context.writer.Write(entry.Name);
+						}
+
+						foreach(SerializationEntry entry in info)
+						{
+							GetWriter(context, entry.ObjectType).WriteTypeTag
+								(context, entry.ObjectType);
+						}
+
+						foreach(SerializationEntry entry in info)
+						{
+							GetWriter(context, entry.ObjectType).WriteTypeSpec
+								(context, entry.ObjectType);
+						}
+						if(assemblyID != -1)
+						{
+							context.writer.Write((int)assemblyID);
+						}
+					}
+					else
+					{
+						// Write a short header, referring to a previous
+						// object's type information.
+						context.writer.Write
+							((byte)(BinaryElementType.RefTypeObject));
+						context.writer.Write((int)objectID);
+						context.writer.Write((int)prevObject);
+					}
+				}
+
+		// Write the object form of values for a type.
+		public override void WriteObject(BinaryValueContext context,
+			Object value, Type type)
+		{
+			StreamingContext streamContext = context.formatter.Context;
+			SerializationInfo info = new SerializationInfo(type, context.formatter.converter);
+			
+			((ISerializable) value).GetObjectData(info, streamContext);
+
+            // the entries are written using the type-spec supplied when
+            // they were put into the SerInfo, but the writer is determined
+            // by the real type!
+			foreach(SerializationEntry entry in info)
+			{
+				Object val = entry.Value;
+				if(val != null)
+				{
+					GetWriter(context, val.GetType()).WriteInline
+						(context, val, val.GetType(), entry.ObjectType);
+				}
+				else
+				{
+                    // NULL is always written as object
+					GetWriter(context, typeof(Object)).WriteInline
+						(context, val, typeof(Object), entry.ObjectType);
+				}
+			}
+		}
+	}
+
 	// Write object values using serialization surrogates.
 	private class SurrogateWriter : ObjectWriter
 	{
@@ -681,7 +820,7 @@ internal abstract class BinaryValueWriter
 	{
 		// Internal state.
 		private BinaryPrimitiveTypeCode code;
-		private String fieldName;
+		protected String fieldName;
 
 		// Constructor.
 		public PrimitiveWriter(BinaryPrimitiveTypeCode code)
@@ -806,7 +945,7 @@ internal abstract class BinaryValueWriter
 										 Object value, Type type,
 										 Type fieldType)
 				{
-					context.writer.Write((ushort)(char)value);
+					context.writer.Write((char)value);
 				}
 
 	}; // class CharWriter
@@ -981,17 +1120,27 @@ internal abstract class BinaryValueWriter
 						context.writer.Write((int)prevObject);
 					}
 				}
+				// Write the object form of values for a type.
+				public override void WriteObject(BinaryValueContext context,
+												 Object value, Type type)
+				{
+					// NOTE: the stupid swapping of the int's is intentional
+					Decimal dec = (Decimal) value;
+					int[] bits = Decimal.GetBits(dec);
+					context.writer.Write(bits[3]);
+					context.writer.Write(bits[2]);
+					context.writer.Write(bits[0]);
+					context.writer.Write(bits[1]);
+				}
 
 		// Write the inline form of values for a type.
 		public override void WriteInline(BinaryValueContext context,
 										 Object value, Type type,
 										 Type fieldType)
 				{
-					int[] bits = Decimal.GetBits((Decimal)value);
-					context.writer.Write(bits[3]);
-					context.writer.Write(bits[2]);
-					context.writer.Write(bits[0]);
-					context.writer.Write(bits[1]);
+					// NOTE: it's stupid ... but Bug for bug compatibility
+					// was much simpler to write the bits out and read it back
+					context.writer.Write(value.ToString());
 				}
 
 	}; // class DecimalWriter
@@ -1003,13 +1152,47 @@ internal abstract class BinaryValueWriter
 		public DateTimeWriter()
 			: base(BinaryPrimitiveTypeCode.DateTime, "ticks") {}
 
+				// Write the object header information for a type.
+		public override void WriteObjectHeader(BinaryValueContext context,
+												Object value, Type type,
+												long objectID, long prevObject)
+				{
+					if(prevObject == -1)
+					{
+						// Write the full type information.
+						context.writer.Write((byte)(BinaryElementType.RuntimeObject));
+						context.writer.Write((int)objectID);
+						context.writer.Write(type.FullName);
+						context.writer.Write((int)1);
+						context.writer.Write(fieldName);
+						int64Writer.WriteTypeTag(context, null);
+						int64Writer.WriteTypeSpec(context, null);
+					}
+					else
+					{
+						// Write a short header, referring to a previous
+						// object's type information.
+						context.writer.Write((byte)(BinaryElementType.RefTypeObject));
+						context.writer.Write((int)objectID);
+						context.writer.Write((int)prevObject);
+					}
+				}
+
+		// Write the object form of values for a type.
+		public override void WriteObject(BinaryValueContext context,
+										 Object value, Type type)
+		{
+			// this has to be saved as long
+			context.writer.Write((long) ((DateTime)value).Ticks);
+		}
+
 		// Write the inline form of values for a type.
 		public override void WriteInline(BinaryValueContext context,
 										 Object value, Type type,
 										 Type fieldType)
-				{
-					context.writer.Write(((DateTime)value).Ticks);
-				}
+		{
+			context.writer.Write(((DateTime)value).Ticks);
+		}
 
 	}; // class DateTimeWriter
 
@@ -1020,13 +1203,47 @@ internal abstract class BinaryValueWriter
 		public TimeSpanWriter()
 			: base(BinaryPrimitiveTypeCode.TimeSpan, "_ticks") {}
 
+				// Write the object header information for a type.
+		public override void WriteObjectHeader(BinaryValueContext context,
+											   Object value, Type type,
+											   long objectID, long prevObject)
+		{
+			if(prevObject == -1)
+			{
+				// Write the full type information.
+				context.writer.Write((byte)(BinaryElementType.RuntimeObject));
+				context.writer.Write((int)objectID);
+				context.writer.Write(type.FullName);
+				context.writer.Write((int)1);
+				context.writer.Write(fieldName);
+				int64Writer.WriteTypeTag(context, null);
+				int64Writer.WriteTypeSpec(context, null);
+			}
+			else
+			{
+				// Write a short header, referring to a previous
+				// object's type information.
+				context.writer.Write((byte)(BinaryElementType.RefTypeObject));
+				context.writer.Write((int)objectID);
+				context.writer.Write((int)prevObject);
+			}
+		}
+
+		// Write the object form of values for a type.
+		public override void WriteObject(BinaryValueContext context,
+										 Object value, Type type)
+		{
+			// this has to be saved as long
+			context.writer.Write((long) ((TimeSpan)value).Ticks);
+		}
+
 		// Write the inline form of values for a type.
 		public override void WriteInline(BinaryValueContext context,
 										 Object value, Type type,
 										 Type fieldType)
-				{
-					context.writer.Write(((TimeSpan)value).Ticks);
-				}
+		{
+			context.writer.Write(((TimeSpan)value).Ticks);
+		}
 
 	}; // class TimeSpanWriter
 
@@ -1039,24 +1256,47 @@ internal abstract class BinaryValueWriter
 		// Write the type tag for a type.
 		public override void WriteTypeTag
 					(BinaryValueContext context, Type type)
-				{
-					context.writer.Write((byte)(BinaryTypeTag.String));
-				}
+		{
+			context.writer.Write((byte)(BinaryTypeTag.String));
+		}
 
 		// Write the type specification for a type.
 		public override void WriteTypeSpec
 					(BinaryValueContext context, Type type)
-				{
-					// Nothing to do here.
-				}
+		{
+			// Nothing to do here.
+		}
 
 		// Write the inline form of values for a type.
 		public override void WriteInline(BinaryValueContext context,
 										 Object value, Type type,
 										 Type fieldType)
+		{
+			bool firstTime;
+			
+			if(value == null)
+			{
+				// Write a null value.
+				context.writer.Write((byte)(BinaryElementType.NullValue));
+				return;
+			} 
+			else 
+			{
+				long objectID = context.gen.GetId(value, out firstTime);
+				if(firstTime) 
 				{
+					context.writer.Write((byte)(BinaryElementType.String));
+					context.writer.Write((int)objectID);
 					context.writer.Write((String)value);
+				} 
+				else 
+				{
+					context.writer.Write
+							((byte)(BinaryElementType.ObjectReference));
+					context.writer.Write((int)objectID);
 				}
+			}
+		}
 
 		// Write the object header information for a type.
 		public override void WriteObjectHeader(BinaryValueContext context,
@@ -1064,13 +1304,23 @@ internal abstract class BinaryValueWriter
 											   long objectID, long prevObject)
 				{
 					context.writer.Write((byte)(BinaryElementType.String));
+					context.writer.Write((int) objectID);
 				}
 
 		// Write the object form of values for a type.
 		public override void WriteObject(BinaryValueContext context,
 										 Object value, Type type)
 				{
-					context.writer.Write((String)value);
+					if(value == null)
+					{
+						// Write a null value.
+						context.writer.Write
+								((byte)(BinaryElementType.NullValue));
+					} 
+					else 
+					{
+						context.writer.Write((String)value);
+					}
 				}
 
 	}; // class StringWriter
@@ -1197,7 +1447,17 @@ internal abstract class BinaryValueWriter
 						}
 						else
 						{
-							// TODO: generic array typespecs
+							bool firstTime;
+							long assemblyId;
+							assemblyId = context.gen.GetId
+							(type.GetElementType().Assembly, out firstTime);
+							context.writer.Write(type.FullName);
+							context.writer.Write((int)assemblyId);
+							if(firstTime)
+							{
+								// We need to output the assembly later.
+								context.assemblyQueue.Enqueue(type.Assembly);
+							}
 						}
 					}
 				}
@@ -1295,13 +1555,36 @@ internal abstract class BinaryValueWriter
 		public override void WriteObject(BinaryValueContext context,
 										 Object value, Type type)
 				{
-					// TODO: write the array contents
+					Array ar = (Array) value;
+					int length = ar.GetLength(0);
+					for(int i = 0; i < length; i++) 
+					{
+						if(ar[i] == null)
+						{
+							// Write a null value.
+							context.writer.Write
+								((byte)(BinaryElementType.NullValue));
+						}
+						else
+						{
+							BinaryValueWriter writer = GetWriter(context, type.GetElementType());
+							if(writer != null)
+							{
+								writer.WriteInline(context, ar[i], ar[i].GetType(), type);
+							}
+							else
+							{
+								throw new SerializationException
+									(String.Format
+										(_("Serialize_CannotSerialize"), type));
+							}
+						}
+					}
 				}
-
 	}; // class ArrayWriter
 
 }; // class BinaryValueWriter
 
-#endif // CONFIG_SERIALIZATION
+//#endif // CONFIG_SERIALIZATION
 
 }; // namespace System.Runtime.Serialization.Formatters.Binary
